package com.faraphel.tasks_valider.connectivity.task

import com.faraphel.tasks_valider.connectivity.task.session.TaskSessionManager
import com.faraphel.tasks_valider.database.TaskDatabase
import fi.iki.elonen.NanoHTTPD


/**
 * A server to handle the task API to allow clients to interact with the database.
 * @param port the port of the server
 * @param database the database to interact with
 */
class TaskServer(
    private val port: Int,
    private val database: TaskDatabase
) : NanoHTTPD(port) {
    private val sessionManager = TaskSessionManager()  ///< the session manager

    override fun serve(httpSession: IHTTPSession): Response {
        // get the session data of the client
        val taskSession = this.sessionManager.getOrCreateSessionData(httpSession)

        // get the method used
        val method: Method = httpSession.method

        // parse the url
        val uri: String = httpSession.uri.substring(1)  // remove the first slash
        val uriComponents = uri.split("/")

        // get the type of the request from the uri
        val requestType = uriComponents.getOrNull(0)
            ?: return newFixedLengthResponse(
                NanoHTTPD.Response.Status.BAD_REQUEST,
                "text/plain",
                "Invalid request type"
            )

        // get the response from the correct part of the application
        val response = when (requestType) {
            // session requests
            "sessions" ->
                this.sessionManager.handleApiRequest(taskSession, httpSession, method)

            // entities requests
            "entities" -> {
                val entityName = uriComponents.getOrNull(1)
                    ?: return newFixedLengthResponse(
                        NanoHTTPD.Response.Status.BAD_REQUEST,
                        "text/plain",
                        "Invalid entity name"
                    )

                // TODO(Faraphel): the uri arguments should be handled in the handleApiRequest
                return this.database.handleApiRequest(taskSession, httpSession, method, entityName)
            }

            // invalid requests
            else ->
                newFixedLengthResponse(
                    NanoHTTPD.Response.Status.BAD_REQUEST,
                    "text/plain",
                    "Invalid request type"
                )
        }

        // wrap additional information in the response
        return this.sessionManager.responseSetSessionData(response, httpSession.cookies)
    }

    /**
     * Start the server with the default configuration
     */
    override fun start() = super.start(SOCKET_READ_TIMEOUT, false)
}
