package com.faraphel.tasks_valider.connectivity.task.session

import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import fi.iki.elonen.NanoHTTPD
import java.util.*


/**
 * The manager for the session system
 */
class TaskSessionManager {
    private val jsonParser = Gson()  ///< the json parser
    private val jsonTypeToken = object : TypeToken<Map<String, String>>(){}  ///< the json type
    private val sessions = mutableMapOf<String, TaskSession>()  ///< sessions specific data

    /**
     * Create a new session
     * @param session the data for the session (optional)
     * @param sessionId the session id to use (optional)
     * @return a new session identifiant
     */
    fun newSessionData(
        session: TaskSession = TaskSession(),
        sessionId: String = UUID.randomUUID().toString()
    ): String {
        this.sessions[sessionId] = session
        return sessionId
    }

    /**
     * Get data from a http session
     * @param httpSession the HTTP session
     * @return the session data
     */
    fun getSessionData(httpSession: NanoHTTPD.IHTTPSession): TaskSession? {
        val sessionId = httpSession.cookies.read("sessionId") ?: return null
        val sessionData = this.getSessionData(sessionId)
        return sessionData
    }

    /**
     * Get data from a session identifiant
     * @param sessionId the identifiant of the session
     * @return the session data
     */
    fun getSessionData(sessionId: String): TaskSession? {
        return this.sessions[sessionId]
    }

    /**
     * Get data from a http session. If it does not exist, create it.
     * @param httpSession the HTTP session
     */
    fun getOrCreateSessionData(httpSession: NanoHTTPD.IHTTPSession): TaskSession {
        // try to get the session directly
        var session = this.getSessionData(httpSession)

        // if the session does not exist, create it
        if (session == null) {
            val sessionId = this.newSessionData()
            session = this.getSessionData(sessionId)!!
        }

        // return the session
        return session
    }

    /**
     * Insert new cookies for a session in a response
     * @param response the response to inject cookies into
     * @param cookies the cookie handler
     */
    fun responseSetSessionData(
        response: NanoHTTPD.Response,
        cookies: NanoHTTPD.CookieHandler
    ): NanoHTTPD.Response {
        cookies.set(NanoHTTPD.Cookie("sessionId", this.newSessionData()))
        cookies.unloadQueue(response)
        return response
    }

    /**
     * Handle a HTTP Api request
     * @param taskSession the data of the client session
     * @param httpSession the data of the http session
     * @param method the method used
     */
    fun handleApiRequest(
        taskSession: TaskSession,
        httpSession: NanoHTTPD.IHTTPSession,
        method: NanoHTTPD.Method
    ): NanoHTTPD.Response {
        when (method) {
            // get a client session data
            NanoHTTPD.Method.GET -> {
                TODO("Faraphel: implement get")
            }
            // change a client session data
            NanoHTTPD.Method.POST -> {
                // check the permission of the session
                if (taskSession.role.permissions.contains(TaskPermission.ADMIN))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.FORBIDDEN,
                        "text/plain",
                        "Forbidden"
                    )

                // parse the content of the request
                val data: Map<String, String> = jsonParser.fromJson(
                    httpSession.inputStream.bufferedReader().readText(),
                    jsonTypeToken.type
                )

                // update the role
                data["role"]?.let { role ->
                    try {
                        taskSession.role = TaskRole.valueOf(role)
                    } catch (exception: IllegalArgumentException) {
                        return NanoHTTPD.newFixedLengthResponse(
                            NanoHTTPD.Response.Status.BAD_REQUEST,
                            "text/plain",
                            "Invalid role"
                        )
                    }
                    // NOTE(Faraphel): does a modification on the object require updating the array ?
                }

                // success message
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.OK,
                    "text/plain",
                    "Session updated"
                )
            }

            // other action are limited
            else -> {
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
                    "text/plain",
                    "Method not allowed"
                )
            }
        }
    }
}